library teamdrt_firebase_package;

import 'dart:async';

import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';
import 'package:google_sign_in/google_sign_in.dart';
import 'package:teamdrt_firebase_package/PackageConsts.dart';

/// Login Methods for Firebase
/// Make sure to initialise FirebaseApp before calling these methods
/// These Services will help you help you implement Login and SignUp methods for firebase

class SignInBloc {
  final FirebaseAuth _auth = FirebaseAuth.instance;

  Stream<User> get user {
    return _auth.authStateChanges();
  }

  var errorController = StreamController<String>.broadcast();
  var signInStatus = StreamController<SignInStatus>.broadcast();

  Future logInWithEmail([
    String email,
    String password,
    ActionCodeSettings actionCodeSettings,
  ]) async {
    try {
      signInStatus.sink.add(SignInStatus.PROCESSING);
      _auth
          .signInWithEmailAndPassword(email: email, password: password)
          .then((value) {
        if (!value.user.emailVerified) {
          value.user.sendEmailVerification(actionCodeSettings);
          signInStatus.sink.add(SignInStatus.EMAILSENT);
        }
      });
    } on FirebaseException catch (e) {
      failed(e.code);
    } on PlatformException catch (e) {
      failed(e.code);
    } on Exception catch (e) {
      failed(e.toString());
    }
  }

  Future signOut() async {
    _auth.signOut();
  }

  Future loginWithGoogle() async {
    try {
      final GoogleSignIn googleSignIn = GoogleSignIn();
      final GoogleSignInAccount googleSignInAccount =
          await googleSignIn.signIn();
      final GoogleSignInAuthentication googleSignInAuthentication =
          await googleSignInAccount.authentication;
      print(googleSignInAuthentication.accessToken);
      final AuthCredential credential = GoogleAuthProvider.credential(
        accessToken: googleSignInAuthentication.accessToken,
        idToken: googleSignInAuthentication.idToken,
      );
      _auth.signInWithCredential(credential);
    } on PlatformException catch (e) {
      failed(e.code);
    } on FirebaseException catch (e) {
      failed(e.code);
    } catch (e) {
      debugPrint(
        e,
      );
      failed(Constants.unknownError);
    }
  }

  failed(String error) {
    print(error);
    errorController.sink.add(error);
    signInStatus.sink.add(SignInStatus.FAILED);
  }

  dispose() {
    errorController.close();
    signInStatus.close();
  }
}

class SignUpBloc {
  final FirebaseAuth _auth = FirebaseAuth.instance;

  Stream<User> get user {
    return _auth.authStateChanges();
  }

  var errorController = StreamController<String>.broadcast();
  var signUpStatus = StreamController<SignInStatus>.broadcast();

  Future signUpWithGoogle() async {
    try {
      signUpStatus.sink.add(SignInStatus.PROCESSING);
      final GoogleSignIn googleSignIn = GoogleSignIn();
      final GoogleSignInAccount googleSignInAccount =
          await googleSignIn.signIn();
      final GoogleSignInAuthentication googleSignInAuthentication =
          await googleSignInAccount.authentication;
      print(googleSignInAuthentication.accessToken);
      final AuthCredential credential = GoogleAuthProvider.credential(
        accessToken: googleSignInAuthentication.accessToken,
        idToken: googleSignInAuthentication.idToken,
      );
      await _auth.signInWithCredential(credential).then((value) {
        if (value.user != null) {
          signUpStatus.sink.add(SignInStatus.COMPLETED);
        }
      });
    } on PlatformException catch (e) {
      failed(e.code);
    } on FirebaseException catch (e) {
      failed(e.code);
    } catch (e) {
      debugPrint(
        e,
      );
      failed(Constants.unknownError);
    }
  }

  Future signUpWithEmailPassword({
    String email,
    String password,
    String number,
    String userName,
    ActionCodeSettings actionCodeSettings,
  }) async {
    try {
      signUpStatus.sink.add(SignInStatus.PROCESSING);
      await _auth
          .createUserWithEmailAndPassword(email: email, password: password)
          .then(
        (value) {
          if (!value.user.emailVerified) {
            value.user.sendEmailVerification(actionCodeSettings);
            signUpStatus.sink.add(SignInStatus.EMAILSENT);
          }
          value.user.updateProfile(
            displayName: userName,
          );
        },
      );
    } on FirebaseException catch (e) {
      failed(e.code);
    } on PlatformException catch (e) {
      failed(e.code);
    } on Exception catch (e) {
      failed(e.toString());
    }
  }

  failed(String error) {
    print(error);
    errorController.sink.add(error);
    signUpStatus.sink.add(SignInStatus.FAILED);
  }

  dispose() {
    errorController.close();
    signUpStatus.close();
  }
}

enum SignInStatus {
  PROCESSING,
  EMAILSENT,
  COMPLETED,
  OTPSENT,
  FAILED,
}
